@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import hashlib
 import re
 
 import glance_store
@@ -45,6 +46,7 @@ CONF.import_opt('disk_formats', 'glance.common.config', group='image_format')
 CONF.import_opt('container_formats', 'glance.common.config',
                 group='image_format')
 CONF.import_opt('show_multiple_locations', 'glance.common.config')
+CONF.import_opt('hashing_algorithm', 'glance.common.config')
 
 
 class ImagesController(object):
@@ -90,15 +92,53 @@ class ImagesController(object):
 
     @utils.mutating
     def import_image(self, req, image_id, body):
+        image_repo = self.gateway.get_repo(req.context)
         task_factory = self.gateway.get_task_factory(req.context)
         executor_factory = self.gateway.get_task_executor_factory(req.context)
         task_repo = self.gateway.get_task_repo(req.context)
-
-        task_input = {'image_id': image_id,
-                      'import_req': body}
-
         import_method = body.get('method').get('name')
         uri = body.get('method').get('uri')
+
+        try:
+            image = image_repo.get(image_id)
+            if image.status == 'active':
+                msg = _("Image with status active cannot be target for import")
+                raise exception.Conflict(msg)
+            if image.status != 'queued' and import_method == 'web-download':
+                msg = _("Image needs to be in 'queued' state to use "
+                        "'web-download' method")
+                raise exception.Conflict(msg)
+            if (image.status != 'uploading' and
+                    import_method == 'glance-direct'):
+                msg = _("Image needs to be staged before 'glance-direct' "
+                        "method can be used")
+                raise exception.Conflict(msg)
+            if not getattr(image, 'container_format', None):
+                msg = _("'container_format' needs to be set before import")
+                raise exception.Conflict(msg)
+            if not getattr(image, 'disk_format', None):
+                msg = _("'disk_format' needs to be set before import")
+                raise exception.Conflict(msg)
+
+            backend = None
+            if CONF.enabled_backends:
+                backend = req.headers.get('x-image-meta-store',
+                                          CONF.glance_store.default_backend)
+                try:
+                    glance_store.get_store_from_store_identifier(backend)
+                except glance_store.UnknownScheme:
+                    msg = _("Store for scheme %s not found") % backend
+                    LOG.warn(msg)
+                    raise exception.Conflict(msg)
+        except exception.Conflict as e:
+            raise webob.exc.HTTPConflict(explanation=e.msg)
+        except exception.NotFound as e:
+            raise webob.exc.HTTPNotFound(explanation=e.msg)
+
+        task_input = {'image_id': image_id,
+                      'import_req': body,
+                      'backend': backend}
+
         if (import_method == 'web-download' and
            not utils.validate_import_uri(uri)):
                 LOG.debug("URI for web-download does not pass filtering: %s",
@@ -142,6 +182,14 @@ class ImagesController(object):
             filters = {}
         filters['deleted'] = False
 
+        os_hidden = filters.get('os_hidden', 'false').lower()
+        if os_hidden not in ['true', 'false']:
+            message = _("Invalid value '%s' for 'os_hidden' filter."
+                        " Valid values are 'true' or 'false'.") % os_hidden
+            raise webob.exc.HTTPBadRequest(explanation=message)
+        # ensure the type of os_hidden is boolean
+        filters['os_hidden'] = os_hidden == 'true'
+
         protected = filters.get('protected')
         if protected is not None:
             if protected not in ['true', 'false']:
@@ -228,10 +276,10 @@ class ImagesController(object):
         path = change['path']
         path_root = path[0]
         value = change['value']
-        if path_root == 'locations' and value == []:
+        if path_root == 'locations' and not value:
             msg = _("Cannot set locations to empty list.")
             raise webob.exc.HTTPForbidden(msg)
-        elif path_root == 'locations' and value != []:
+        elif path_root == 'locations' and value:
             self._do_replace_locations(image, value)
         elif path_root == 'owner' and req.context.is_admin == False:
             msg = _("Owner can't be updated by non admin.")
@@ -287,11 +335,12 @@ class ImagesController(object):
         try:
             image = image_repo.get(image_id)
 
-            # NOTE(abhishekk): If 'image-import' is supported and image status
-            # is uploading then delete image data from the staging area.
-            if CONF.enable_image_import and image.status == 'uploading':
+            if image.status == 'uploading':
                 file_path = str(CONF.node_staging_uri + '/' + image.image_id)
-                self.store_api.delete_from_backend(file_path)
+                if CONF.enabled_backends:
+                    self.store_api.delete(file_path, None)
+                else:
+                    self.store_api.delete_from_backend(file_path)
 
             image.delete()
             image_repo.remove(image)
@@ -317,6 +366,76 @@ class ImagesController(object):
         except exception.NotAuthenticated as e:
             raise webob.exc.HTTPUnauthorized(explanation=e.msg)
 
+    def _validate_validation_data(self, image, locations):
+        val_data = {}
+        for loc in locations:
+            if 'validation_data' not in loc:
+                continue
+            for k, v in loc['validation_data'].items():
+                if val_data.get(k, v) != v:
+                    msg = _("Conflicting values for %s") % k
+                    raise webob.exc.HTTPConflict(explanation=msg)
+                val_data[k] = v
+
+        # NOTE(imacdonn): values may be provided for items which are
+        # already set, so long as the values exactly match. In this
+        # case, nothing actually needs to be updated, but we should
+        # reject the request if there's an apparent attempt to supply
+        # a different value.
+        new_val_data = {}
+        for k, v in val_data.items():
+            current = getattr(image, k)
+            if v == current:
+                continue
+            if current:
+                msg = _("%s is already set with a different value") % k
+                raise webob.exc.HTTPConflict(explanation=msg)
+            new_val_data[k] = v
+
+        if not new_val_data:
+            return {}
+
+        if image.status != 'queued':
+            msg = _("New value(s) for %s may only be provided when image "
+                    "status is 'queued'") % ', '.join(new_val_data.keys())
+            raise webob.exc.HTTPConflict(explanation=msg)
+
+        if 'checksum' in new_val_data:
+            try:
+                checksum_bytes = bytearray.fromhex(new_val_data['checksum'])
+            except ValueError:
+                msg = (_("checksum (%s) is not a valid hexadecimal value") %
+                       new_val_data['checksum'])
+                raise webob.exc.HTTPConflict(explanation=msg)
+            if len(checksum_bytes) != 16:
+                msg = (_("checksum (%s) is not the correct size for md5 "
+                         "(should be 16 bytes)") %
+                       new_val_data['checksum'])
+                raise webob.exc.HTTPConflict(explanation=msg)
+
+        hash_algo = new_val_data.get('os_hash_algo')
+        if hash_algo != CONF['hashing_algorithm']:
+            msg = (_("os_hash_algo must be %(want)s, not %(got)s") %
+                   {'want': CONF['hashing_algorithm'], 'got': hash_algo})
+            raise webob.exc.HTTPConflict(explanation=msg)
+
+        try:
+            hash_bytes = bytearray.fromhex(new_val_data['os_hash_value'])
+        except ValueError:
+            msg = (_("os_hash_value (%s) is not a valid hexadecimal value") %
+                   new_val_data['os_hash_value'])
+            raise webob.exc.HTTPConflict(explanation=msg)
+        want_size = hashlib.new(hash_algo).digest_size
+        if len(hash_bytes) != want_size:
+            msg = (_("os_hash_value (%(value)s) is not the correct size for "
+                     "%(algo)s (should be %(want)d bytes)") %
+                   {'value': new_val_data['os_hash_value'],
+                    'algo': hash_algo,
+                    'want': want_size})
+            raise webob.exc.HTTPConflict(explanation=msg)
+
+        return new_val_data
+
     def _get_locations_op_pos(self, path_pos, max_pos, allow_max):
         if path_pos is None or max_pos is None:
             return None
@@ -340,10 +459,16 @@ class ImagesController(object):
                     "%s.") % image.status
             raise webob.exc.HTTPConflict(explanation=msg)
 
+        val_data = self._validate_validation_data(image, value)
+
         try:
             # NOTE(flwang): _locations_proxy's setattr method will check if
             # the update is acceptable.
             image.locations = value
+            if image.status == 'queued':
+                for k, v in val_data.items():
+                    setattr(image, k, v)
+                image.status = 'active'
         except (exception.BadStoreUri, exception.DuplicateLocation) as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except ValueError as ve:    # update image status failed.
@@ -361,6 +486,8 @@ class ImagesController(object):
                     "%s.") % image.status
             raise webob.exc.HTTPConflict(explanation=msg)
 
+        val_data = self._validate_validation_data(image, [value])
+
         pos = self._get_locations_op_pos(path_pos,
                                          len(image.locations), True)
         if pos is None:
@@ -369,6 +496,8 @@ class ImagesController(object):
         try:
             image.locations.insert(pos, value)
             if image.status == 'queued':
+                for k, v in val_data.items():
+                    setattr(image, k, v)
                 image.status = 'active'
         except (exception.BadStoreUri, exception.DuplicateLocation) as e:
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
@@ -413,12 +542,13 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
     _disallowed_properties = ('direct_url', 'self', 'file', 'schema')
     _readonly_properties = ('created_at', 'updated_at', 'status', 'checksum',
                             'size', 'virtual_size', 'direct_url', 'self',
-                            'file', 'schema', 'id')
+                            'file', 'schema', 'id', 'os_hash_algo',
+                            'os_hash_value')
     _reserved_properties = ('location', 'deleted', 'deleted_at')
     _base_properties = ('checksum', 'created_at', 'container_format',
                         'disk_format', 'id', 'min_disk', 'min_ram', 'name',
                         'size', 'virtual_size', 'status', 'tags', 'owner',
-                        'updated_at', 'visibility', 'protected')
+                        'updated_at', 'visibility', 'protected', 'os_hidden')
     _available_sort_keys = ('name', 'status', 'container_format',
                             'disk_format', 'size', 'id', 'created_at',
                             'updated_at')
@@ -701,7 +831,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
     def _get_filters(self, filters):
         visibility = filters.get('visibility')
         if visibility:
-            if visibility not in ['community', 'public', 'private', 'shared']:
+            if visibility not in ['community', 'public', 'private', 'shared',
+                                  'all']:
                 msg = _('Invalid visibility value: %s') % visibility
                 raise webob.exc.HTTPBadRequest(explanation=msg)
         changes_since = filters.get('changes-since')
@@ -822,9 +953,6 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
     def import_image(self, request):
-        if not CONF.enable_image_import:
-            msg = _("Image import is not supported at this site.")
-            raise webob.exc.HTTPNotFound(explanation=msg)
         body = self._get_request_body(request)
         self._validate_import_body(body)
         return {'body': body}
@@ -854,7 +982,8 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             attributes = ['name', 'disk_format', 'container_format',
                           'visibility', 'size', 'virtual_size', 'status',
                           'checksum', 'protected', 'min_ram', 'min_disk',
-                          'owner']
+                          'owner', 'os_hidden', 'os_hash_algo',
+                          'os_hash_value']
             for key in attributes:
                 image_view[key] = getattr(image, key)
             image_view['id'] = image.image_id
@@ -894,6 +1023,20 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             image_view['file'] = self._get_image_href(image, 'file')
             image_view['schema'] = '/v2/schemas/image'
             image_view = self.schema.filter(image_view)  # domain
+
+            # add store information to image
+            if CONF.enabled_backends:
+                locations = _get_image_locations(image)
+                if locations:
+                    stores = []
+                    for loc in locations:
+                        backend = loc['metadata'].get('backend')
+                        if backend:
+                            stores.append(backend)
+
+                    if stores:
+                        image_view['stores'] = ",".join(stores)
+
             return image_view
         except exception.Forbidden as e:
             raise webob.exc.HTTPForbidden(explanation=e.msg)
@@ -902,15 +1045,17 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         response.status_int = http.CREATED
         self.show(response, image)
         response.location = self._get_image_href(image)
-        # TODO(rosmaita): remove the outer 'if' statement when the
-        # enable_image_import config option is removed
-        if CONF.enable_image_import:
-            # according to RFC7230, headers should not have empty fields
-            # see http://httpwg.org/specs/rfc7230.html#field.components
-            if CONF.enabled_import_methods:
-                import_methods = ("OpenStack-image-import-methods",
-                                  ','.join(CONF.enabled_import_methods))
-                response.headerlist.append(import_methods)
+        # according to RFC7230, headers should not have empty fields
+        # see http://httpwg.org/specs/rfc7230.html#field.components
+        if CONF.enabled_import_methods:
+            import_methods = ("OpenStack-image-import-methods",
+                              ','.join(CONF.enabled_import_methods))
+            response.headerlist.append(import_methods)
+
+        if CONF.enabled_backends:
+            enabled_backends = ("OpenStack-image-store-ids",
+                                ','.join(CONF.enabled_backends.keys()))
+            response.headerlist.append(enabled_backends)
 
     def show(self, response, image):
         image_view = self._format_image(image)
@@ -980,12 +1125,30 @@ def get_base_properties():
             'type': 'boolean',
             'description': _('If true, image will not be deletable.'),
         },
+        'os_hidden': {
+            'type': 'boolean',
+            'description': _('If true, image will not appear in default '
+                             'image list response.'),
+        },
         'checksum': {
             'type': ['null', 'string'],
             'readOnly': True,
             'description': _('md5 hash of image contents.'),
             'maxLength': 32,
         },
+        'os_hash_algo': {
+            'type': ['null', 'string'],
+            'readOnly': True,
+            'description': _('Algorithm to calculate the os_hash_value'),
+            'maxLength': 64,
+        },
+        'os_hash_value': {
+            'type': ['null', 'string'],
+            'readOnly': True,
+            'description': _('Hexdigest of the image contents using the '
+                             'algorithm specified by the os_hash_algo'),
+            'maxLength': 128,
+        },
         'owner': {
             'type': ['null', 'string'],
             'description': _('Owner of the image'),
@@ -1060,6 +1223,11 @@ def get_base_properties():
             'readOnly': True,
             'description': _('An image file url'),
         },
+        'backend': {
+            'type': 'string',
+            'readOnly': True,
+            'description': _('Backend store to upload image to'),
+        },
         'schema': {
             'type': 'string',
             'readOnly': True,
@@ -1077,6 +1245,36 @@ def get_base_properties():
                     'metadata': {
                         'type': 'object',
                     },
+                    'validation_data': {
+                        'description': _(
+                            'Values to be used to populate the corresponding '
+                            'image properties. If the image status is not '
+                            '\'queued\', values must exactly match those '
+                            'already contained in the image properties.'
+                        ),
+                        'type': 'object',
+                        'writeOnly': True,
+                        'additionalProperties': False,
+                        'properties': {
+                            'checksum': {
+                                'type': 'string',
+                                'minLength': 32,
+                                'maxLength': 32,
+                            },
+                            'os_hash_algo': {
+                                'type': 'string',
+                                'maxLength': 64,
+                            },
+                            'os_hash_value': {
+                                'type': 'string',
+                                'maxLength': 128,
+                            },
+                        },
+                        'required': [
+                            'os_hash_algo',
+                            'os_hash_value',
+                        ],
+                    },
                 },
                 'required': ['url', 'metadata'],
             },
