@@ -25,16 +25,21 @@ import os
 import sys
 
 import eventlet
-from oslo_utils import encodeutils
-
-# Monkey patch socket and time
 # NOTE(jokke): As per the eventlet commit
 # b756447bab51046dfc6f1e0e299cc997ab343701 there's circular import happening
 # which can be solved making sure the hubs are properly and fully imported
 # before calling monkey_patch(). This is solved in eventlet 0.22.0 but we
 # need to address it before that is widely used around.
 eventlet.hubs.get_hub()
-eventlet.patcher.monkey_patch(all=False, socket=True, time=True, thread=True)
+
+if os.name == 'nt':
+    # eventlet monkey patching the os module causes subprocess.Popen to fail
+    # on Windows when using pipes due to missing non-blocking IO support.
+    eventlet.patcher.monkey_patch(os=False)
+else:
+    eventlet.patcher.monkey_patch()
+
+from oslo_utils import encodeutils
 
 # If ../glance/__init__.py exists, add ../ to Python search path, so that
 # it will override what happens to be installed in /usr/(local/)lib/python...
@@ -55,6 +60,7 @@ from glance import notifier
 CONF = cfg.CONF
 CONF.import_group("profiler", "glance.common.wsgi")
 logging.register_options(CONF)
+wsgi.register_cli_opts()
 
 
 def main():
