@@ -128,7 +128,7 @@ def cooperative_read(fd):
 MAX_COOP_READER_BUFFER_SIZE = 134217728  # 128M seems like a sane buffer limit
 
 CONF.import_group('import_filtering_opts',
-                  'glance.async.flows._internal_plugins')
+                  'glance.async_.flows._internal_plugins')
 
 
 def validate_import_uri(uri):
@@ -218,7 +218,7 @@ class CooperativeReader(object):
                 result = self.buffer[self.position:]
                 self.buffer = b''
                 self.position = 0
-                return str(result)
+                return bytes(result)
             else:
                 # otherwise read the next chunk from the underlying iterator
                 # and return it as a whole. Reset the buffer, as subsequent
@@ -228,7 +228,7 @@ class CooperativeReader(object):
                         self.iterator = self.__iter__()
                     return next(self.iterator)
                 except StopIteration:
-                    return ''
+                    return b''
                 finally:
                     self.buffer = b''
                     self.position = 0
@@ -271,20 +271,23 @@ class LimitingReader(object):
     Reader designed to fail when reading image data past the configured
     allowable amount.
     """
-    def __init__(self, data, limit):
+    def __init__(self, data, limit,
+                 exception_class=exception.ImageSizeLimitExceeded):
         """
         :param data: Underlying image data object
         :param limit: maximum number of bytes the reader should allow
+        :param exception_class: Type of exception to be raised
         """
         self.data = data
         self.limit = limit
         self.bytes_read = 0
+        self.exception_class = exception_class
 
     def __iter__(self):
         for chunk in self.data:
             self.bytes_read += len(chunk)
             if self.bytes_read > self.limit:
-                raise exception.ImageSizeLimitExceeded()
+                raise self.exception_class()
             else:
                 yield chunk
 
@@ -292,7 +295,7 @@ class LimitingReader(object):
         result = self.data.read(i)
         self.bytes_read += len(result)
         if self.bytes_read > self.limit:
-            raise exception.ImageSizeLimitExceeded()
+            raise self.exception_class()
         return result
 
 
