@@ -225,11 +225,14 @@ def _image_format(image_id, **values):
         'size': None,
         'virtual_size': None,
         'checksum': None,
+        'os_hash_algo': None,
+        'os_hash_value': None,
         'tags': [],
         'created_at': dt,
         'updated_at': dt,
         'deleted_at': None,
         'deleted': False,
+        'os_hidden': False
     }
 
     locations = values.pop('locations', None)
@@ -258,6 +261,7 @@ def _filter_images(images, filters, context,
         status = None
 
     visibility = filters.pop('visibility', None)
+    os_hidden = filters.pop('os_hidden', False)
 
     for image in images:
         member = image_member_find(context, image_id=image['id'],
@@ -267,6 +271,7 @@ def _filter_images(images, filters, context,
         image_is_public = image['visibility'] == 'public'
         image_is_community = image['visibility'] == 'community'
         image_is_shared = image['visibility'] == 'shared'
+        image_is_hidden = image['os_hidden'] == True
         acts_as_admin = context.is_admin and not admin_as_user
         can_see = (image_is_public
                    or image_is_community
@@ -299,6 +304,10 @@ def _filter_images(images, filters, context,
             if not image_is_public == is_public:
                 continue
 
+        if os_hidden:
+            if image_is_hidden:
+                continue
+
         to_add = True
         for k, value in six.iteritems(filters):
             key = k
@@ -474,6 +483,17 @@ def image_get_all(context, filters=None, marker=None, limit=None,
     return res
 
 
+def image_restore(context, image_id):
+    """Restore the pending-delete image to active."""
+    image = _image_get(context, image_id)
+    if image['status'] != 'pending_delete':
+        msg = (_('cannot restore the image from %s to active (wanted '
+                 'from_state=pending_delete)') % image['status'])
+        raise exception.Conflict(msg)
+    values = {'status': 'active', 'deleted': 0}
+    image_update(context, image_id, values)
+
+
 @log_call
 def image_property_create(context, values):
     image = _image_get(context, values['image_id'])
@@ -716,7 +736,8 @@ def image_create(context, image_values, v1_mode=False):
                         'virtual_size', 'checksum', 'locations', 'owner',
                         'protected', 'is_public', 'container_format',
                         'disk_format', 'created_at', 'updated_at', 'deleted',
-                        'deleted_at', 'properties', 'tags', 'visibility'])
+                        'deleted_at', 'properties', 'tags', 'visibility',
+                        'os_hidden', 'os_hash_algo', 'os_hash_value'])
 
     incorrect_keys = set(image_values.keys()) - allowed_keys
     if incorrect_keys:
