@@ -315,11 +315,16 @@ class NotificationBase(object):
     def get_payload(self, obj):
         return {}
 
-    def send_notification(self, notification_id, obj, extra_payload=None):
+    def send_notification(self, notification_id, obj, extra_payload=None,
+                          backend=None):
         payload = self.get_payload(obj)
         if extra_payload is not None:
             payload.update(extra_payload)
 
+        # update backend information in the notification
+        if backend:
+            payload["backend"] = backend
+
         _send_notification(self.notifier.info, notification_id, payload)
 
 
@@ -389,8 +394,8 @@ class ImageProxy(NotificationProxy, domain_proxy.Image):
             'bytes_sent': bytes_sent,
             'image_id': self.repo.image_id,
             'owner_id': self.repo.owner,
-            'receiver_tenant_id': self.context.tenant,
-            'receiver_user_id': self.context.user,
+            'receiver_tenant_id': self.context.project_id,
+            'receiver_user_id': self.context.user_id,
         }
 
     def _get_chunk_data_iterator(self, data, chunk_size=None):
@@ -419,12 +424,12 @@ class ImageProxy(NotificationProxy, domain_proxy.Image):
         data = self.repo.get_data(offset=offset, chunk_size=chunk_size)
         return self._get_chunk_data_iterator(data, chunk_size=chunk_size)
 
-    def set_data(self, data, size=None):
-        self.send_notification('image.prepare', self.repo)
+    def set_data(self, data, size=None, backend=None):
+        self.send_notification('image.prepare', self.repo, backend=backend)
 
         notify_error = self.notifier.error
         try:
-            self.repo.set_data(data, size)
+            self.repo.set_data(data, size, backend=backend)
         except glance_store.StorageFull as e:
             msg = (_("Image storage media is full: %s") %
                    encodeutils.exception_to_unicode(e))
@@ -443,7 +448,7 @@ class ImageProxy(NotificationProxy, domain_proxy.Image):
             raise webob.exc.HTTPBadRequest(
                 explanation=encodeutils.exception_to_unicode(e))
         except exception.Duplicate as e:
-            msg = (_("Unable to upload duplicate image data for image"
+            msg = (_("Unable to upload duplicate image data for image "
                      "%(image_id)s: %(error)s") %
                    {'image_id': self.repo.image_id,
                     'error': encodeutils.exception_to_unicode(e)})
