@@ -22,14 +22,16 @@ import shlex
 import shutil
 import socket
 import subprocess
+import threading
 
 from alembic import command as alembic_command
 import fixtures
+import mock
 from oslo_config import cfg
 from oslo_config import fixture as cfg_fixture
+from oslo_log.fixture import logging_error as log_fixture
 from oslo_log import log
 from oslo_serialization import jsonutils
-from oslotest import moxstubout
 import six
 from six.moves import BaseHTTPServer
 from six.moves import http_client as http
@@ -76,9 +78,7 @@ class BaseTestCase(testtools.TestCase):
         # the following policy tests
         config.parse_args(args=[])
         self.addCleanup(CONF.reset)
-        mox_fixture = self.useFixture(moxstubout.MoxStubout())
-        self.stubs = mox_fixture.stubs
-        self.stubs.Set(exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
+        self.mock_object(exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
         self.test_dir = self.useFixture(fixtures.TempDir()).path
         self.conf_dir = os.path.join(self.test_dir, 'etc')
         utils.safe_mkdirs(self.conf_dir)
@@ -87,6 +87,10 @@ class BaseTestCase(testtools.TestCase):
         # Limit the amount of DeprecationWarning messages in the unit test logs
         self.useFixture(glance_fixtures.WarningsFixture())
 
+        # Make sure logging output is limited but still test debug formatting
+        self.useFixture(log_fixture.get_logging_handle_error_fixture())
+        self.useFixture(glance_fixtures.StandardLogging())
+
     def set_policy(self):
         conf_file = "policy.json"
         self.policy_file = self._copy_data_file(conf_file, self.conf_dir)
@@ -134,9 +138,21 @@ class BaseTestCase(testtools.TestCase):
         """
         self._config_fixture.config(**kw)
 
+    def mock_object(self, obj, attr_name, *args, **kwargs):
+        """"Use python mock to mock an object attirbute
+
+        Mocks the specified objects attribute with the given value.
+        Automatically performs 'addCleanup' for the mock.
+        """
+        patcher = mock.patch.object(obj, attr_name, *args, **kwargs)
+        result = patcher.start()
+        self.addCleanup(patcher.stop)
+        return result
+
 
 class requires(object):
     """Decorator that initiates additional test setup/teardown."""
+
     def __init__(self, setup=None, teardown=None):
         self.setup = setup
         self.teardown = teardown
@@ -155,12 +171,17 @@ class requires(object):
 
 class depends_on_exe(object):
     """Decorator to skip test if an executable is unavailable"""
+
     def __init__(self, exe):
         self.exe = exe
 
     def __call__(self, func):
         def _runner(*args, **kw):
-            cmd = 'which %s' % self.exe
+            if os.name != 'nt':
+                cmd = 'which %s' % self.exe
+            else:
+                cmd = 'where.exe', '%s' % self.exe
+
             exitcode, out, err = execute(cmd, raise_error=False)
             if exitcode != 0:
                 args[0].disabled_message = 'test requires exe: %s' % self.exe
@@ -309,7 +330,11 @@ def execute(cmd,
     path_ext = [os.path.join(os.getcwd(), 'bin')]
 
     # Also jack in the path cmd comes from, if it's absolute
-    args = shlex.split(cmd)
+    if os.name != 'nt':
+        args = shlex.split(cmd)
+    else:
+        args = cmd
+
     executable = args[0]
     if os.path.isabs(executable):
         path_ext.append(os.path.dirname(executable))
@@ -468,7 +493,7 @@ def start_http_server(image_id, image_data):
                 self.send_response(http.OK)
                 self.send_header('Content-Length', str(len(fixture)))
                 self.end_headers()
-                self.wfile.write(fixture)
+                self.wfile.write(six.b(fixture))
                 return
 
             def do_HEAD(self):
@@ -494,11 +519,11 @@ def start_http_server(image_id, image_data):
     httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
     port = httpd.socket.getsockname()[1]
 
-    pid = os.fork()
-    if pid == 0:
-        httpd.serve_forever()
-    else:
-        return pid, port
+    thread = threading.Thread(target=httpd.serve_forever)
+    thread.daemon = True
+    thread.start()
+
+    return thread, httpd, port
 
 
 class RegistryAPIMixIn(object):
@@ -694,3 +719,28 @@ def is_sqlite_version_prior_to(major, minor):
     import sqlite3
     tup = sqlite3.sqlite_version_info
     return tup[0] < major or (tup[0] == major and tup[1] < minor)
+
+
+def start_standalone_http_server():
+    def _get_http_handler_class():
+        class StaticHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
+            def do_GET(self):
+                data = b"Hello World!!!"
+                self.send_response(http.OK)
+                self.send_header('Content-Length', str(len(data)))
+                self.end_headers()
+                self.wfile.write(data)
+                return
+
+        return StaticHTTPRequestHandler
+
+    server_address = ('127.0.0.1', 0)
+    handler_class = _get_http_handler_class()
+    httpd = BaseHTTPServer.HTTPServer(server_address, handler_class)
+    port = httpd.socket.getsockname()[1]
+
+    thread = threading.Thread(target=httpd.serve_forever)
+    thread.daemon = True
+    thread.start()
+
+    return thread, httpd, port
